/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Berlin2 BG2Q clock tree IDs
 */

#define CLKID_SYS		0
#define CLKID_DRMFIGO		1
#define CLKID_CFG		2
#define CLKID_GFX2D		3
#define CLKID_ZSP		4
#define CLKID_PERIF		5
#define CLKID_PCUBE		6
#define CLKID_VSCOPE		7
#define CLKID_NFC_ECC		8
#define CLKID_VPP		9
#define CLKID_APP		10
#define CLKID_SDIO0XIN		11
#define CLKID_SDIO1XIN		12
#define CLKID_GFX2DAXI		13
#define CLKID_GETH0		14
#define CLKID_SATA		15
#define CLKID_AHBAPB		16
#define CLKID_USB0		17
#define CLKID_USB1		18
#define CLKID_USB2		19
#define CLKID_USB3		20
#define CLKID_PBRIDGE		21
#define CLKID_SDIO		22
#define CLKID_NFC		23
#define CLKID_SMEMC		24
#define CLKID_PCIE		25
#define CLKID_TWD		26
#define CLKID_CPU		27
